BUNDLE_GEMFILE=Gemfile.rails-5.2 bundle update
BUNDLE_GEMFILE=Gemfile.rails-5.1 bundle update
BUNDLE_GEMFILE=Gemfile.rails-5.0 bundle update
BUNDLE_GEMFILE=Gemfile.rails-4.2 bundle update
BUNDLE_GEMFILE=Gemfile.rails-4.1 bundle update
BUNDLE_GEMFILE=Gemfile.rails-4.0 bundle update
BUNDLE_GEMFILE=Gemfile.mongoid-9.0 bundle update
BUNDLE_GEMFILE=Gemfile.mongoid-8.0 bundle update
BUNDLE_GEMFILE=Gemfile.mongoid-7.0 bundle update
BUNDLE_GEMFILE=Gemfile.mongoid-6.0 bundle update
BUNDLE_GEMFILE=Gemfile.mongoid-5.0 bundle update
BUNDLE_GEMFILE=Gemfile.mongoid-4.0 bundle update
